"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var raygun_transport_1 = require("./raygun.transport");
var timer_1 = require("./timer");
var types_1 = require("./types");
var debug = require("debug")("raygun");
exports.MAX_MESSAGES_IN_BATCH = 100;
exports.MAX_BATCH_SIZE_BYTES = 1638400;
var MAX_BATCH_INNER_SIZE_BYTES = exports.MAX_BATCH_SIZE_BYTES - 2; // for the starting and ending byte
var RaygunBatchTransport = /** @class */ (function () {
    function RaygunBatchTransport(options) {
        this.timerId = null;
        this.batchId = 0;
        this.batchState = { messages: [], messageSizeInBytes: 0 };
        this.interval = options.interval;
        this.httpOptions = options.httpOptions;
    }
    RaygunBatchTransport.prototype.send = function (options) {
        var _this = this;
        this.onIncomingMessage({
            serializedMessage: options.message,
            callback: options.callback,
        });
        if (!this.timerId) {
            this.timerId = setTimeout(function () { return _this.processBatch(); }, 1000);
        }
    };
    RaygunBatchTransport.prototype.stopProcessing = function () {
        if (this.timerId) {
            debug("batch transport - stopping");
            clearInterval(this.timerId);
            this.timerId = null;
        }
    };
    RaygunBatchTransport.prototype.onIncomingMessage = function (messageAndCallback) {
        var serializedMessage = messageAndCallback.serializedMessage, callback = messageAndCallback.callback;
        var messageLength = Buffer.byteLength(serializedMessage, "utf-8");
        if (messageLength >= MAX_BATCH_INNER_SIZE_BYTES) {
            var messageSize = Math.ceil(messageLength / 1024);
            var startOfMessage = serializedMessage.slice(0, 1000);
            console.warn("[raygun4node] Error is too large to send to Raygun (" + messageSize + "kb)\nStart of error: " + startOfMessage);
            return;
        }
        var messageIsTooLargeToAddToBatch = this.batchState.messageSizeInBytes + messageLength >
            MAX_BATCH_INNER_SIZE_BYTES;
        if (messageIsTooLargeToAddToBatch) {
            this.processBatch();
        }
        if (this.batchState.messages.length === 0) {
            this.batchState.messageSizeInBytes += messageLength;
        }
        else {
            this.batchState.messageSizeInBytes += messageLength + 1; // to account for the commas between items
        }
        this.batchState.messages.push(messageAndCallback);
        var batchIsFull = this.batchState.messages.length === 100;
        if (batchIsFull) {
            this.processBatch();
        }
    };
    RaygunBatchTransport.prototype.processBatch = function () {
        var payload = this.batchState.messages
            .map(function (m) { return m.serializedMessage; })
            .join(",");
        var batch = {
            payload: "[" + payload + "]",
            messageCount: this.batchState.messages.length,
            callbacks: this.batchState.messages.map(function (m) { return m.callback; }),
        };
        this.sendBatch(batch);
        this.batchState = { messages: [], messageSizeInBytes: 0 };
        this.stopProcessing();
    };
    RaygunBatchTransport.prototype.sendBatch = function (batch) {
        var payload = batch.payload, messageCount = batch.messageCount, callbacks = batch.callbacks;
        debug("batch transport - processing ( " + messageCount + " message(s) in batch)");
        var batchId = this.batchId;
        this.batchId++;
        var runAllCallbacks = function (err, response) {
            var durationInMs = stopTimer();
            if (err) {
                debug("batch transport - error sending batch (id=" + batchId + ", duration=" + durationInMs + "ms): " + err);
            }
            else {
                debug("batch transport - successfully sent batch (id=" + batchId + ", duration=" + durationInMs + "ms)");
            }
            for (var _i = 0, callbacks_1 = callbacks; _i < callbacks_1.length; _i++) {
                var callback = callbacks_1[_i];
                if (callback) {
                    types_1.callVariadicCallback(callback, err, response);
                }
            }
        };
        debug("batch transport - sending batch (id=" + batchId + ") (" + messageCount + " messages, " + payload.length + " bytes)");
        var stopTimer = timer_1.startTimer();
        raygun_transport_1.sendBatch({
            message: payload,
            callback: runAllCallbacks,
            http: this.httpOptions,
        });
    };
    return RaygunBatchTransport;
}());
exports.RaygunBatchTransport = RaygunBatchTransport;
